<?php

namespace Drupal\uw_cfg_common\Service;

use Drupal\Core\Entity\EntityInterface;
use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;

/**
 * Class UWService
 *
 * @package Drupal\uw_cfg_common\Service
 */
class UWService implements UWServiceInterface {

  /**
   * {@inheritDoc}
   */
  public function prepareResponsiveImage(EntityInterface $entity, string $image_style): array {

    // Load in the file object if we have one.
    if ($file = $entity->field_media_image->entity) {

      // Need to set these variables so that responsive image function,
      // has all the necessary info to process the image style.
      $variables['uri'] = $file->getFileUri();
      $variables['responsive_image_style_id'] = $image_style;

      // These is a function from the responsive image module that sets all
      // the variables for the sources of the responsive image.
      template_preprocess_responsive_image($variables);

      // Step through each of the sources and setup our own sources array.
      foreach ($variables['sources'] as $source) {
        $variables['responsive_sources'][] = [
          'srcset' => $source->storage()['srcset']->value(),
          'media' => $source->storage()['media']->value(),
          'type' => $source->storage()['type']->value(),
        ];
      }

      return $variables;
    }

    return [];
  }

  /**
   * {@inheritDoc}
   */
  public function getTeaserContent(NodeInterface $node, array $variables_to_get, string $teaser_type): array {
    $variables = [];
    // Step through each of the variables to get and set the variables for the
    // teaser content.
    foreach ($variables_to_get as $variable_to_get) {

      // Switch on the variable to get and set the variables for the teaser
      // content.
      switch ($variable_to_get) {

        case 'title':

          // Set the title from the node object.
          $variables['title'] = $node->getTitle();
          break;

        case 'url':

          // Set the title from the node object.
          $variables['url'] = $node->toUrl()->toString();
          break;

        case 'date':

          // Set the field name, all of our content types use the same format,
          // field_uw_<content_type>_date.
          $field_name = 'field_uw_' . trim($teaser_type) . '_date';

          // Set the date variable, once returned to the calling function, they
          // can change the date format as required (i.e. change it to long-date
          // or date-time).
          $variables['date'] = $node->$field_name->getString();
          break;

        case 'author':

          // If author is selected, get both the author name and link.
          $variables['author_name'] = $node->getOwner()->getDisplayName();
          $variables['author_link'] = $node->getOwner()->toUrl()->toString();
          break;

        case 'sources':

          // Event has a different name for the image than the rest, so ensuring
          // that we get the correct field name for the image.
          // In most cases it is field_uw_<content_type>_lising_page_image.
          if ($teaser_type == 'event') {
            $field_name = 'field_uw_event_listing_page_img';
          }
          else {
            $field_name = 'field_uw_' . trim($teaser_type) . '_listing_page_image';
          }

          // Get the image object from the node.
          $image = $node->$field_name->entity;

          // Ensure that we have an image before adding variables.
          if ($image !== NULL) {

            // Get all the image variables, including the sources using the
            // prepareResponsiveImage function.
            $image_variables = $this->prepareResponsiveImage($image, 'uw_ris_media');

            // Set the responsive image variables for the teaser content.
            $variables['sources'] = $image_variables['responsive_sources'];
            $variables['img_element'] = $image_variables['img_element']['#uri'];
            $variables['alt'] = $image->field_media_image->alt;
          }
          break;

        case 'tags':

          // Set the field name for the tags which is
          // field_uw_<content_type>_tags.
          $field_name = 'field_uw_' . trim($teaser_type) . '_tags';

          // Get all the taxonomy terms for the blog.
          // Step through each taxonomy term and get the tag info.
          foreach ($node->$field_name as $tag) {

            // Set the tags in the teaser content variables.
            $variables['tags'][] = [
              'title' => $tag->entity->name->value,
              'url' => $tag->entity->toUrl()->toString(),
            ];
          }
          break;

        case 'content':

          // Set the field name for the summary, which is
          // field_name<content_type>_summary.
          $field_name = 'field_uw_' . $teaser_type . '_summary';

          // Set the render array for the summary, we simply can not just send
          // the value as Drupal will just render it with no filters or HTML
          // applied. We need to send the full render array.
          $variables['content'] = [
            '#type' => 'processed_text',
            '#text' => $node->$field_name->value,
            '#format' => $node->$field_name->format,
          ];
          break;
      }
    }

    return $variables;
  }

}
